@@ -638,12 +638,16 @@ def removeFileWithEmptyDirectory(rmName, stopDirectory="/"):
                                          stopDirectory=stopDirectory)
 
 
-def getRunCommands(not_chroot=False, chroot=None):
+def getRunCommands(not_chroot=False, chroot=None, uid=None):
     """List run program"""
 
     def getCmd(procNum):
         cmdLineFile = '/proc/%s/cmdline' % procNum
         try:
+            if uid is not None:
+                fstat = os.stat("/proc/%s"%procNum)
+                if fstat.st_uid != uid:
+                    return ""
             if path.exists(cmdLineFile):
                 if not_chroot:
                     root_link = '/proc/%s/root' % procNum
@@ -799,18 +803,26 @@ def makeDirectory(pathname):
         return False
 
 
-def readLinesFile(filename):
+def readLinesFile(filename, grab=False):
     """Read file by line"""
     try:
         if path.exists(filename):
             for line in open(filename, 'r'):
-                yield line.rstrip('\n')
+                if grab:
+                    line = line.strip()
+                    if not line.startswith("#"):
+                        yield line
+                else:
+                    yield line.rstrip('\n')
     except (OSError, IOError):
         pass
     finally:
         raise StopIteration
 
 
+def isEmptyFile(filename, grab=False):
+    return not any(readLinesFile(filename, grab=grab))
+
 def readFile(filename, tailbyte=None, headbyte=None):
     """
     Прочитать целый файл или вернуть пустую строку.
@@ -1465,3 +1477,86 @@ def tar_directory(dn, archfile):
                 tar.add(x, path.basename(x))
     except (IOError, OSError) as e:
         raise FilesError(_("Failed to create tarball: %s") % str(e))
+
+
+def tar_list(file_list, archfile, prefix='/'):
+    """
+    Заархивировать указанный список
+    :param file_list:
+    :param archfile:
+    :param prefix:
+    :return:
+    """
+    try:
+        with tarfile.open(archfile, "w:bz2") as tar:
+            for fn in (pathJoin(prefix, x) for x in file_list):
+                tar.add(fn, path.relpath(fn, prefix))
+    except (IOError, OSError) as e:
+        raise FilesError(_("Failed to create tarball: %s") % str(e))
+
+
+def tar_unpack(archfile, target_dn):
+    """
+    Распаковать архив в указанную директорию
+    :param archfile:
+    :param target_dn:
+    :return:
+    """
+    try:
+        with tarfile.open(archfile, "r:bz2") as tar:
+            for item in tar:
+                tar.extract(item, target_dn)
+                print item.path, item.mtime
+                os.utime(path.join(target_dn, item.path),
+                         (item.mtime, item.mtime))
+    except (tarfile.TarError, IOError, OSError) as e:
+        raise FilesError(_("Failed to create tarball: %s") % str(e))
+
+class RsyncOptions(object):
+    Xattrs = "--xattrs"
+    KeepDirLinks = "--keep-dirlinks"
+    OneFileSystem = "--one-file-system"
+    Archive = "--archive"
+    FilesFromStdin = "--files-from=-"
+
+    # опции используемые при "скрывании пакета"
+    # --keep-dirlinks решает проблему восстановления "скрытых" файлов
+    HidePackageOpts = (Xattrs, KeepDirLinks, OneFileSystem, Archive)
+
+def rsync_files(source_dn, target_dn, *file_list, **kw):
+    """
+    Перенести файлы file_list из prefix в target_dn с сохранением путей и
+    аттрибутов
+    :param file_list:
+    :param target_dn:
+    :param prefix:
+    :return:
+    """
+    opts = kw.get('opts', ())
+    rsync = getProgPath('/usr/bin/rsync')
+    source_dn = "%s/" % path.normpath(source_dn)
+    target_dn = "%s/" % path.normpath(target_dn)
+    if file_list:
+        params = list(opts) + [RsyncOptions.FilesFromStdin, source_dn,
+                               target_dn]
+    else:
+        params = list(opts) + [source_dn, target_dn]
+    p = process(rsync, *params, stderr=PIPE)
+    if file_list:
+        for fn in file_list:
+            p.write("%s\n" % fn)
+    if p.failed():
+        raise FilesError(_("Failed to rsync {src} to {dst}: {error}").format(
+            src=source_dn, dst=target_dn,
+            error=p.readerr()))
+
+def xz(data, decompress=False):
+    xz_prog = getProgPath('/usr/bin/xz')
+    p = Popen([xz_prog, "-dc" if decompress else "-zc"],
+              stdin=PIPE, stderr=PIPE, stdout=PIPE, close_fds=True)
+    data, errors = p.communicate(input=data)
+    p.poll()
+    if p.returncode == 0:
+        return data
+    else:
+        raise FilesError(errors)
